#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Copyright © 2004-2005 Guido Amoruso

This file is part of xml2ly

xml2ly is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

xml2ly is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""


from musicxml import *

from xml.sax import make_parser
from xml.sax.handler import *

import string
import os
import sys


def unicode2int (u):
	try:
		return int (str (u))
	except ValueError:
		return u
	

class MusicXMLContentHandler (ContentHandler):
	def __init__ (self):
		self.element = ''
		self.in_element = False
		self.elements = []

		self.tree = {}
		self.store_tree = False
		self.nodes = []

		self.score = None
		self.identification = None
		self.work = None
		self.part = None
		self.voice = None
		self.note = None
		self.direction = None
		self.divisions = None
		self.time = None
		self.clef = None
		self.backup = 0
		self.forward = 0
		
		self.moment = 0

	def startElement (self, name, attrs):
		self.element = name
		self.in_element = True
		self.elements.append (name)
		
		if self.store_tree:
			if not self.tree.has_key (name):
				self.tree[name] = []

			attributes = attrs.__dict__['_attrs']
			attributes = dict (zip (attributes.keys (), map (unicode2int, attributes.values ())))
			self.tree[name].append ({})
			#self.tree[name][-1].update (attrs.__dict__['_attrs'])
			self.tree[name][-1].update (attributes)

			self.nodes.append (self.tree)
			self.tree = self.tree[name][-1]

		elif name == 'score-partwise':
			self.score = Score ()

		elif name == 'score-timewise':
			raise Exception ('ScoreTimewiseUnsupportedError')
			
		elif name == 'part':
			self.part = Part (attrs.get ('id'))

		elif name == 'measure':
			self.measure = Measure (int (str (attrs['number'])))
			self.moment = 0
			
		elif name in ['identification', 'work', \
			      'note', 'attributes', 'direction', 'barline', \
			      'backup', 'forward']:
			attributes = attrs.__dict__['_attrs']
			attributes = dict (zip (attributes.keys (), map (unicode2int, attributes.values ())))
			self.tree = attributes
			#self.tree = attrs.__dict__['_attrs']

			if name == 'identification':
				self.identification = self.tree
			if name == 'work':
				self.work = self.tree
			elif name == 'note':
				self.note = self.tree
			elif name == 'attributes':
				self.attributes = self.tree
			elif name == 'direction':
				self.directions = self.tree
			elif name == 'barline':
				self.barline = self.tree
			elif name == 'backup':
				self.backup = self.tree
			elif name == 'forward':
				self.forward = self.tree
				
			self.nodes.append (self.tree)
			self.store_tree = True


	def characters (self, content):
		if self.element == '' and self.in_element == True:
			pass

		elif self.store_tree and self.in_element:
			if not content.isspace ():
				parent = self.nodes[-1]
				if not parent.has_key (self.element):
					parent[self.element] = []
				try:
					content = int (str (content))
				except ValueError:
					content = content
				parent[self.element][-1] = content


	def endElement (self, name):
		self.element = name
		self.in_element = False
		self.elements.pop ()
		event = None
		events = []

		if name in ['note', 'attributes', 'direction', 'barline']:
			self.store_tree = False

			if name == 'note':
				event = NoteFactory (self.note, self.moment)

				if 'chord' not in self.note and 'duration' in self.note:
					self.moment += self.note['duration'][0]
					
			elif name == 'attributes':
				events = AttributesFactory (self.attributes, self.moment)

				for e in events:
					if isinstance (e, Divisions):
						self.divisions = e
					elif isinstance (e, Time):
						self.time = e
					elif isinstance (e, Clef):
						self.clef = e
						
			elif name == 'direction':
				events = DirectionFactory (self.directions, self.moment)
				
			elif name == 'barline':
				event = BarlineFactory (self.barline, self.moment)

			if event:
				event.measure = self.measure
				event.divisions = self.divisions
				event.time = self.time
				event.clef = self.clef

				self.part.add_event (event)

			if events:
				for event in events:
					event.measure = self.measure
					event.divisions = self.divisions
					event.time = self.time
					event.clef = self.clef
					
					self.part.add_event (event)

		elif name in ['backup', 'forward']:
			self.store_tree = False

			if name == 'backup':
				self.moment -= self.backup['duration'][0]

			elif name == 'forward':
				self.moment += self.forward['duration'][0]

		elif name == 'measure':
			pass
		
		elif name in ['work']:
			self.store_tree = False

			self.score.work = Work (self.work)
		
		elif name in ['identification']:
			self.store_tree = False

			self.score.identification = Identification (self.identification)

		elif name == 'part':
			self.score.add_part (self.part)

		elif name == 'score-partwise':
			pass

		elif self.store_tree:
			self.tree = self.nodes.pop ()


def main ():
	parser = make_parser ()
	handler = MusicXMLContentHandler ()
	parser.setContentHandler (handler)
	parser.setFeature (feature_external_ges, False)

	parser.parse (open (sys.argv[1]))


if __name__ == '__main__':
	main ()


